% Vamos a generar muestras de trfico sinttico a partir de unos parametros
% que introducira el usuario.

function generadora(total_peticiones,porcen_arch_dist,porcen_one_timer,...
    pend_zipf,pareto_tail_index,porcen_en_cola,k,meanlognorm,...
    stdlognorm,correlacion,tam_pila,modo_pila,tipo,traficotxt)

warning off;

arch_dist = porcen_arch_dist*total_peticiones/100; %En los comentarios = a
one_timer = porcen_one_timer*arch_dist/100;        %En los comentarios = b




% --------------- GENERAMOS POPULARIDAD (LEY ZIPF) ----------------------

% La ley zipf dice:  P = cte / r^beta
% debemos encontrar el valor de la constante cte, para ello utilizamos el 
% onetimer mas centrado ya que sabemos que solo ocurre una vez, es decir,
% conocemos su probabilidad, y su ranking ya que sabemos donde estan
% ubicados los one timers (al final), de este modo 1 = cte / (a-b/2)^beta
% por lo que cte=(a-b/2)^beta y de este modo podremos generar todas las
% probabilidades como:
%      Pr = cte/r^beta  valido para las a-b primeras peticiones distintas

const_zipf = (arch_dist - one_timer/2)^pend_zipf;     % Hallamos cte
ranking = single(linspace(1,arch_dist-one_timer,arch_dist-one_timer))';
popularidad = single(ones(arch_dist,1));
% Hallamos la popularidad siguiendo ley Zipf
popularidad=const_zipf./(ranking.^pend_zipf);    
 % Los NO one timers tienen como minimo popularidad 2
popularidad(popularidad<2)=2;                 
 % vamos a asignar popularidad 1 a los one timers
popularidad(arch_dist-one_timer+1:arch_dist,1)=1;   
 % redondeamos al entero ms proximo
popularidad=round(popularidad);                     

% Puede ocurrir que nos pasemos, es decir que la suma de popularidades sea
% mayor que el numero total de peticiones a generar, por lo que debemos de
% reajustar valores

pop_total=sum(popularidad);

if (pop_total~=total_peticiones)
    factorscala = (total_peticiones-one_timer)/...
                              (sum(popularidad(1:arch_dist-one_timer)));
    popularidad = popularidad.*factorscala;
    popularidad(popularidad<2)=2;                  
    popularidad(arch_dist-one_timer+1:arch_dist,1)=1;     
    popularidad=round(popularidad);                       
end

% ---------------- GENERAMOS DISTRIBUCIONES DE TAMAO ---------------------


% Lo convertimos a distribucion normal
medianorm = log((stdlognorm^2)/(sqrt(stdlognorm^2+meanlognorm^2)));  
stdnorm = sqrt(log((stdlognorm^2+meanlognorm^2)/meanlognorm^2));

arch_cola = round(porcen_en_cola * arch_dist / 100);

i=1;
filesize = single(zeros (arch_dist,1));

% distribucion log-normal (cuerpo)
while i<= arch_dist - arch_cola
    % Esto lo ponemos para que la generacion del numero sea totalmente
    % aleatorio
%     randn('state',sum(100*clock));  
    % Generamos numero aleatorio segun distribucion normal
    varnorm = randn(1,1);           
    tamano=round(exp(medianorm+stdnorm*varnorm));

    % Esta distribucion lognormal debe cumplir que los tamaos sean menores
    % que k ya que si son mayores que k estarian en la cola y seguirian una
    % distribucion de Pareto. k nos marca el limite superior de la 
    % distribucion lognormal

    if tamano <= k
        filesize(i,1)=tamano;
        i=i+1;
    end
end

% distribucion de Pareto (cola)
while i <= arch_dist
    % Generamos numero aleatorio segun distribucion uniforme
    varunif = rand(1,1);           
    tamano=round(k/varunif^(1/pareto_tail_index));
    
 % Establecemos un limite superior de 50 MB como tamao maximo
 % y el valor de k como limite inferior
 % Esta restriccion superior es OPCIONAL y solo vale para cuando
 % simulemos un trafico real
 
    if ( (tamano > k))% && (tamano < 50*1024*1024) )   
        filesize(i,1)=tamano;                        
        i=i+1;                                       
    end                                              
end


% ---------- GENERAMOS CORRELACION TAMAO / POPULARIDAD  ----------------

% correlacion = input ('Introduce correlacion tamao / popularidad: ');

% Generamos lista de popularidad ordenada en orden ascendente
pop_asc = single(sortrows (popularidad));

% Generamos lista de tamaos ordenada en orden ascendente
filesize_asc = single(sortrows (filesize));

acum_pop=(1:length(pop_asc))';
acum_pop=cumsum(acum_pop)./sum(acum_pop);
 % Lista formada por vi,pi(popularidad,probalidad acumulada de la posicion)

 % [numtam,numtamrep]=mmrepeat(filesize_asc);
 
acum_size=(1:length(filesize_asc))';
acum_size=cumsum(acum_size)./sum(acum_size);
 
% Lista formada por vi,pi(tamao,probabilidad aculumalada de la posicion)

lnew=single(zeros(length(filesize_asc),2));

if correlacion==0
  lnew=correlacero(arch_dist,pop_asc,filesize_asc,lnew,acum_pop,acum_size);
elseif correlacion<0
  lnew=correlaneg(arch_dist,pop_asc,filesize_asc,lnew,acum_pop,acum_size);
else
  lnew=correlapos(arch_dist,pop_asc,filesize_asc,lnew,acum_pop,acum_size);
end

% normalizamos para ajustar la popularidad de los id al numero total de
% peticiones a generar. Pero vamos a seguir manteniendo el numero de
% one_timers como aquel que se ha introducido como parametro, eludiendo el
% hecho de que al elegirlos aleatoriamente segun su probabilidad acumulada
% saldran muchos menos one-timers, ya que suponen un % menor sobre el
% numero total de peticiones.

lnew=sortrows(lnew,-1); %Ordenamos descendentemente
if (sum(lnew(:,1))~=total_peticiones)
    factorscala = (total_peticiones-one_timer)/(sum(lnew(:,1))-one_timer);
    lnew(:,1) =round(lnew(:,1).*factorscala);
     % Los NO one timers tienen como minimo popularidad 2
    lnew(lnew(:,1)<2)=2;        
     % vamos a asignar popularidad 1 a los one timers
    lnew(arch_dist-one_timer+1:arch_dist,1)=1;   
end
  
total_peticiones=sum(lnew(:,1));

% Debemos ajustar tambien el tamao de los archivos ya que como
% consecuencia de la interpolacin se han visto aumentados, eso s, la
% relacion entre los tamaos sigue siendo la misma.

factorscala=sum(filesize)/sum(lnew(:,2));
lnew(:,2)=round(lnew(:,2).*factorscala);



% Con lo que hemos generado hasta ahora lo siguiente: Una tabla donde la
% posicion indica el numero de id, el primer campo la popularidad de dicho
% id, y el segundo campo su tamao.
% ---------------- ASIGNAMOS EL TIPO DE TRAFICO A CADA ID ------------

% Jugamos con la tabla hasta tener: id,tamao,tipo, popularidad y % de pop
id=linspace(1,arch_dist,arch_dist)';
lnew(:,3)=id;
lnew(:,4)=tipo*ones(length(id),1);

temp=zeros(length(lnew),1);
temp=lnew(:,4);
lnew(:,4)=lnew(:,1);
lnew(:,1)=lnew(:,3);
lnew(:,3)=temp;
lnew=round(lnew);  % Ojo que la columna 4 ya es de popularidad
lnew(:,5)=lnew(:,4)./sum(lnew(:,4));


% ---------------- GENERAMOS LOCALIDAD TEMPORAL  ---------------------

% Creamos una pila con 6 campos: id, tamao, tipo,popu., % de popu y %
% pop.acumulado de las posiciones
pila=single(zeros(tam_pila+1,6)); 

 % En el caso estatico asignamos las probabilidades acumuladas de los 1000
 % primeros
if modo_pila==1            
    popu_ord = sortrows(lnew(:,5),-1);
    pila(1:tam_pila,6)=cumsum(popu_ord(1:tam_pila));
end
pila(tam_pila+1,:)=single([0;0;0;0;0;0]);

% Una vez que generamos el numero aleatorio debemos ver si la pila esta
% vacia o no, en caso de que no lo est debemos averiguar si esta dentro
% de la pila o no lo esta.
% Crearemos una variable no_elem_pila que nos dira el numero de elementos
% que hay en la pila.

restantes = single(lnew);
no_elem_pila = 1 ;     % Iniciamos a uno el numero de elementos de la pila
no_docs_dist = arch_dist;   % Inicializamos el numero de archivos distintos
total_generadas =1;
trafico=single(zeros(total_peticiones,3));
pet=single(zeros(1,5));     % Aqui almacenaremos la peticion en curso

while total_generadas<total_peticiones
    % Esto lo ponemos para que la generacion del numero sea totalmente
    % aleatorio
%      rand('state',sum(100*clock));  
     % Generamos numero aleatorio segun distribucion uniforme
     no_azar = single(rand(1,1));           

     % Una vez que generamos el numero aleatorio comprobamos si cae dentro
     % del rango de probabilidades acumuladas de la pila, en caso
     % afirmativo cogeremos el id del que ms se aproxime y actuaremos en
     % consecuencia, en caso contrario cogeremos un id del grupo de los
     % restantes y lo meteremos en la pila

     % Cuando la pila esta llena y llega un nuevo id del
     % grupo de los restantes, el ultimo que este en la pila se pierde y es
     % introducido de nuevo en el grupo de los restantes, en la ultima
     % posicion

   %  if busqbin2(no_azar,pila(1:no_elem_pila-1,6))<=no_elem_pila-1
    
     
     
     
 if ( (no_elem_pila>1) && ( (no_docs_dist==0) ||...
                                    (no_azar<=pila(no_elem_pila-1,6)) ) )
       if  (no_docs_dist==0) 
           no_azar=no_azar*pila(no_elem_pila-1,6);
       end
         %  Encontramos el primer elemento mayor que no_azar
       indice=single(busqbin2(no_azar,pila(1:no_elem_pila-1,6)));
         %   indice=find(pila(1:no_elem_pila-1,6)>=no_azar,1);
       trafico(total_generadas,1:3)=pila(indice,1:3);
       total_generadas=single(total_generadas+1);
         % Le restamos 1 a su popularidad
       pila(indice,4)=pila(indice,4)-1;
         % Hallamos su nuevo % de popularidad (creo q no hace falta
         % actualizar pq en caso estatico es siempre el mismo y en caso
         % dinamico se actualizaran todas mas adelante
    % pila(indice,5)=pila(indice,4)/(total_peticiones+1-total_generadas);  
       if pila(indice,4)>0  % Si todavia quedan mas peticiones de este id
           pet=pila(indice,1:5);
             % Desplazamos hacia abajo desde el 1 hasta el id encontrado
           pila(2:indice,1:5)=pila(1:indice-1,1:5);
           pila(1,1:5)=pet;  % La metemos en la primera posicion de pila
       else      % Si ya no hay mas peticiones, la sacamos de la pila
           pila(indice:no_elem_pila-2,1:5)=pila(indice+1:no_elem_pila-1,1:5);
           no_elem_pila=no_elem_pila-1;
       end
       if (modo_pila == 2)   % Recalculamos %'s en caso de modo dinamico
               pila(:,5)=pila(:,4)./(total_peticiones+1-total_generadas);
               if (no_elem_pila >0)
                   pila(:,6)=cumsum(pila(:,5));
               end
       end
 else
     if no_docs_dist>0
         % Lo metemos en el rango de las peticiones restantes
       indice = floor(no_azar*no_docs_dist); % si no furula poner round
       indice(indice<1)=1;
       trafico(total_generadas,1:3) = restantes(indice,1:3);
       total_generadas = total_generadas + 1;
       pet=restantes(indice,:);
        % Metemos la ultima en el hueco de la ya generada
       restantes(indice,:)=restantes(no_docs_dist,:); 
       no_docs_dist = no_docs_dist -1;
       pet(1,4)=pet(1,4)-1;   % Le restamos 1 a su popularidad
       % Hallamos su nuevo % de popularidad (creo q no hace falta
       % actualizar ya que en caso estatico es siempre el mismo y en caso
       % dinamico se actualizaran todas mas adelante
       % pet(1,5)=pet(1,4)/(total_peticiones+1-total_generadas); 
       if pet(1,4)>0        % Si todavia quedan mas peticiones de este id
           if no_elem_pila == tam_pila+1  
               % Si la pila esta llena metemos el ultimo en los restantes
               restantes(no_docs_dist+1,1:5)=pila(tam_pila,1:5);
               no_docs_dist=no_docs_dist+1;
               % Desplazamos hacia abajo desde el 1 hasta donde este el
               % id encontrado
               pila(2:no_elem_pila-1,1:5)=pila(1:no_elem_pila-2,1:5);
               no_elem_pila=no_elem_pila-1; %Hemos quitado un elemento
           else
               % Desplazamos hacia abajo desde el 1 hasta el final
               pila(2:no_elem_pila,1:5)=pila(1:no_elem_pila-1,1:5);    
           end
           pila(1,1:5)=pet(1,1:5); %Metemos la pet en la primera posicion
           no_elem_pila=no_elem_pila +1;
           if (modo_pila ==2)   % Recalculamos en caso de modo dinamico
               pila(:,5)=pila(:,4)./(total_peticiones+1-total_generadas);
               if (no_elem_pila >0)
                   pila(:,6)=cumsum(pila(:,5));
               end
           end
       end
     end
 end
end
archivo=[traficotxt '.txt'];
        fid = fopen(archivo,'wt');
        fprintf(fid,'%8g  %12g  %4g\n',trafico');
        fclose(fid);



